
# q-expansion of newform 273.2.p.f, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 273
weight = 2
poly_data = [16, -32, 32, 160, 320, 80, 0, -8, 60, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[-956416, -156757872, -6794816, -4189816, 1884316, -11267844, 479716, -745616, 28834, -140579, 5381, -12309], 54445816], [[13274488, 262576728, 93750596, 58720980, -23444608, 74667404, -5541268, 10325938, -400085, 1270592, -60738, 170557], 54445816], [[9887728, -69115624, -288671368, -223768168, -46962688, 11897532, -30830234, -33375518, -24618, 119552, -583343, -556438], 54445816], [[8903008, -7918288, -51309608, -199641288, -45708008, -2447648, 11897532, -35281738, 10762, -24618, 119552, -583343], 54445816], [[-22588976, 159724320, -256163344, 487489968, 152924728, 146207356, -197893628, 144220708, -400372, 2016198, -3033606, 2412741], 108891632], [[62241040, -227943584, 255016704, 952913392, 427286336, -74231220, -67932876, 205726644, 2509892, -1302018, -676828, 3446659], 108891632], [[26709024, -23754864, -153928824, -571700956, -137124024, -7342944, 35692596, -105845214, 32286, -73854, 358656, -1750029], 54445816], [[-316463600, -399110384, 443625424, 1688398144, 469375944, -61660468, -111309124, 334136424, 239104, -1166686, -1112876, 5569761], 108891632], [[58170888, -52760848, -328183716, -937941424, -274186516, -13524756, 67292332, -196850238, -134391, -135251, 676169, -3299703], 54445816], [[119102928, -834676912, -1734207216, -2667267136, -619540376, -61660468, -2417492, -482550816, 239104, -1166686, -1112876, -8041693], 108891632]]

hecke_ring_character_values = [[92, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [157, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [106, [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0], [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0], [1, 1, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0], [-1, -1, 0, 0, -1, -1, 0, 1, 0, -1, -1, 0], [0, 1, 0, 0, 0, 0, 1, -1, 0, 0, 0, 0], [0, 1, -1, 1, 1, -1, 0, 1, 0, 1, 0, 1], [-2, -2, 0, -1, 0, -2, 0, 2, 0, -2, -1, 0], [0, 0, -2, 0, 0, 0, -1, 1, 0, 1, 0, 1], [0, 0, -1, 0, -2, 0, 0, 0, -1, 0, 0, 2], [-2, -1, 0, -2, 0, -1, 0, 2, 0, -2, -2, 0], [2, 2, 0, 0, -2, 0, 0, 0, 0, 0, 0, 0], [1, 2, 0, 4, -1, 0, 1, -1, 0, 3, 0, 3], [-3, -1, 0, 0, 3, 0, -2, 2, 0, -2, 0, -2], [0, -2, 0, -2, 0, 2, 0, 0, 0, 0, 2, 0], [3, 0, 0, 0, 3, 1, 1, 1, 4, 2, 0, -2], [-2, 0, 0, 0, 0, 0, 0, 0, 0, -2, 0, 0], [1, 0, 0, 0, 1, 3, 1, 1, 0, 2, 0, -2], [0, 2, -3, 3, 0, -2, 0, 0, -3, 0, -3, 4], [-2, 0, 0, 0, -2, 0, -2, -2, 0, 0, 0, 0], [0, 0, 0, 0, 0, 5, -1, -1, 0, 2, 4, -2], [-3, 0, 0, 0, -3, 4, 0, 0, 0, -1, 2, 1], [4, 3, 1, 1, 0, 3, 0, -2, -1, 4, 1, 0], [1, -1, 0, 2, -1, 0, 3, -3, 0, 0, 0, 0], [-1, 0, 0, 0, -1, -5, 0, 0, -4, -4, 0, 4], [-1, -2, 4, -4, 1, 0, -1, 1, 0, -2, 0, -2], [4, 3, 0, 1, 0, 3, 0, -2, 0, 6, 1, 0], [-8, 0, 1, 1, 0, 0, 0, -2, -1, 2, 1, 0], [-2, 0, 1, 0, 0, 0, 0, -4, -1, 2, 0, 0], [1, 0, 0, 0, 1, 2, 2, 2, 4, 0, 4, 0], [-2, -3, 0, 0, 0, -3, 0, 6, 0, -4, 0, 0], [0, -3, -1, -1, -4, 3, -2, 0, -1, 0, 1, -4], [0, 1, 4, 2, -2, -1, 0, 0, 4, 0, -2, 0], [0, 1, 2, -4, 0, 0, -3, 3, 0, -4, 0, -4], [0, -4, 3, 1, -4, 4, 0, 0, 3, 0, -1, -4], [-2, 0, 0, 0, -2, -3, 3, 3, 0, -2, -4, 2], [-2, 0, -2, -2, 2, 0, -2, 2, 0, 0, 0, 0], [0, -2, -2, -4, -2, 2, -4, 0, -2, 0, 4, -6], [-2, 0, 0, 4, 2, 0, -1, 1, 0, 1, 0, 1], [3, 0, 0, 0, 3, -3, 1, 1, 4, -2, 2, 2], [-8, -5, 4, -5, 0, -5, 0, 0, -4, -10, -5, 0], [0, -1, -3, -2, 6, 1, -6, 0, -3, 0, 2, 2], [8, 6, 0, 0, 0, 6, 0, -2, 0, 6, 0, 0], [14, -1, -1, 0, 0, -1, 0, -2, 1, 4, 0, 0], [-1, -4, 6, -6, 1, 0, 1, -1, 0, -7, 0, -7], [6, 0, 0, 0, 6, 9, -3, -3, 2, 8, 8, -8], [6, 2, 0, 2, 0, 2, 0, -6, 0, 0, 2, 0], [-4, -1, -5, 1, 0, -1, 0, 2, 5, 4, 1, 0], [2, 0, 0, -2, -2, 0, 1, -1, 0, 1, 0, 1], [-3, -1, -4, 2, 3, 0, -1, 1, 0, -2, 0, -2], [3, 0, 0, 0, 3, 0, -2, -2, 4, 7, 2, -7], [0, -3, -4, 2, 2, 3, -6, 0, -4, 0, -2, 0], [-8, 0, 0, 0, -8, 1, -1, -1, 4, 2, -4, -2], [3, 0, 0, 0, 3, -8, 2, 2, 0, -1, -2, 1], [-2, -3, 0, -4, 0, -3, 0, 8, 0, -4, -4, 0], [-4, -1, 0, -5, 0, -1, 0, -2, 0, -6, -5, 0], [-2, 2, -1, -2, 0, 2, 0, 0, 1, 0, -2, 0], [0, -5, -2, -3, -4, 5, -4, 0, -2, 0, 3, 2], [2, 0, 0, 0, 2, -4, -2, -2, -2, 2, 2, -2], [0, -2, 0, -4, 6, 2, 2, 0, 0, 0, 4, -6], [-4, -1, -6, 4, 4, 0, -3, 3, 0, 4, 0, 4], [2, -4, 0, -4, 0, -4, 0, 4, 0, -2, -4, 0], [-1, 0, 0, 0, -1, -3, -4, -4, 0, -4, 2, 4], [0, 0, 0, 0, 0, 10, 1, 1, 2, 1, 4, -1], [6, 3, 0, 0, 0, 3, 0, -6, 0, 8, 0, 0], [0, -8, -3, -7, 10, 8, -8, 0, -3, 0, 7, -6], [2, 0, 0, 0, 2, -3, 5, 5, 6, 4, 0, -4], [-4, 0, 0, 0, -4, -8, 5, 5, 2, -3, -2, 3], [0, -7, 7, -3, 0, 7, -2, 0, 7, 0, 3, -8], [2, 1, -3, 0, 0, 1, 0, -2, 3, -6, 0, 0], [7, 0, 0, 0, 7, 0, -5, -5, 2, -2, 2, 2], [-3, 7, -4, 0, 3, 0, -4, 4, 0, -2, 0, -2], [2, -13, -2, -12, -2, 0, -3, 3, 0, -8, 0, -8], [0, 6, 5, 1, 2, -6, 6, 0, 5, 0, -1, 4], [-6, 3, -3, 3, 0, 3, 0, -2, 3, 0, 3, 0], [-6, 0, 0, 0, -6, 4, 2, 2, -6, -4, 2, 4], [9, -5, 4, -2, -9, 0, -3, 3, 0, -6, 0, -6], [0, -6, 2, -6, -6, 6, 0, 0, 2, 0, 6, -12], [-5, 0, 0, 0, -5, -18, 8, 8, 4, -9, -6, 9], [-12, -1, 8, 0, 12, 0, 1, -1, 0, -6, 0, -6], [3, -2, 0, 8, -3, 0, 3, -3, 0, -2, 0, -2], [0, 7, -2, 2, -2, -7, 4, 0, -2, 0, -2, -4], [-9, 0, 0, 0, -9, -10, 1, 1, -6, -11, -10, 11], [-6, 0, 0, 0, -6, 1, -5, -5, -6, 2, 4, -2], [-6, -2, 5, -1, 0, -2, 0, 2, -5, 0, -1, 0], [2, 4, -1, 5, 0, 4, 0, -4, 1, -2, 5, 0], [-6, 0, 3, -2, 0, 0, 0, -4, -3, -8, -2, 0], [2, -9, -2, -8, -2, 0, -3, 3, 0, 0, 0, 0], [15, 0, 0, 0, 15, 10, -6, -6, 0, 4, 4, -4], [-1, 3, -4, 8, 1, 0, -2, 2, 0, 10, 0, 10], [-2, -4, -4, -4, 2, 0, -2, 2, 0, 0, 0, 0], [-6, -1, -6, -4, 0, -1, 0, 4, 6, 4, -4, 0], [-3, 0, 0, 0, -3, 7, -5, -5, 4, 4, 4, -4], [-4, 0, 0, 0, -4, -8, -2, -2, 2, -4, -2, 4], [0, -2, 1, -2, 18, 2, 4, 0, 1, 0, 2, 2], [-6, 0, 0, 0, -6, -2, 7, 7, -8, -3, -4, 3], [0, -5, -4, 0, -6, 5, -8, 0, -4, 0, 0, -4], [3, -1, 8, 0, -3, 0, 4, -4, 0, 0, 0, 0], [0, 1, -4, 3, -4, -1, 4, 0, -4, 0, -3, 10], [0, 8, -5, 3, -8, -8, 6, 0, -5, 0, -3, 10], [9, 6, 2, -2, -9, 0, -3, 3, 0, -1, 0, -1], [-16, -2, 4, 0, 0, -2, 0, 4, -4, 0, 0, 0], [2, 3, 0, 8, -2, 0, -3, 3, 0, 0, 0, 0], [0, -2, -6, -2, 0, -2, 0, 4, 6, 0, -2, 0], [0, 7, -4, 4, -18, -7, 2, 0, -4, 0, -4, 14], [0, 7, -7, 5, 0, -7, -2, 0, -7, 0, -5, 16], [-5, -2, -8, -2, 5, 0, 2, -2, 0, 5, 0, 5], [-5, 1, -8, 6, 5, 0, -5, 5, 0, 4, 0, 4], [-1, 0, 0, 0, -1, -9, -2, -2, -12, -4, -2, 4], [6, 8, 5, 6, 0, 8, 0, -12, -5, 6, 6, 0], [0, -2, 3, 5, -6, 2, 14, 0, 3, 0, -5, 0], [0, -8, 1, -5, 10, 8, -4, 0, 1, 0, 5, -14], [9, 0, 0, 0, 9, 2, 0, 0, -2, 10, 2, -10], [-2, 0, 0, 0, -2, 13, 3, 3, 4, 6, 0, -6], [6, 0, 0, 0, 6, -4, 6, 6, -2, 2, 2, -2], [-2, -2, 6, 2, 2, 0, -1, 1, 0, -5, 0, -5], [0, -2, -4, 0, 14, 2, 4, 0, -4, 0, 0, 4], [8, 22, 0, 6, -8, 0, 7, -7, 0, 13, 0, 13], [0, 4, 2, -4, 0, 4, 0, 6, -2, -4, -4, 0], [-6, 11, -10, 8, 0, 11, 0, 2, 10, 12, 8, 0], [-6, 3, 5, 6, 0, 3, 0, -10, -5, -4, 6, 0], [-3, 0, 0, 0, -3, 4, 9, 9, 6, 6, 2, -6], [0, -6, -4, -4, -8, 6, -8, 0, -4, 0, 4, -4], [0, 8, 2, 3, -10, -8, 14, 0, 2, 0, -3, 14], [8, 5, 4, 0, -8, 0, 1, -1, 0, 4, 0, 4], [-8, 8, -4, 4, 8, 0, -2, 2, 0, 0, 0, 0], [0, -6, -2, 4, 10, 6, 0, 0, -2, 0, -4, -6], [-5, -12, -6, -6, 5, 0, -4, 4, 0, 4, 0, 4], [10, -3, -4, 0, 0, -3, 0, 0, 4, 8, 0, 0], [-8, -10, 8, -6, 0, -10, 0, 10, -8, -18, -6, 0], [-7, -12, -2, -8, 7, 0, -8, 8, 0, -5, 0, -5], [-2, 12, -4, 8, 2, 0, 2, -2, 0, 8, 0, 8], [2, 0, 0, 0, 2, -19, 5, 5, 2, -4, -12, 4], [0, 2, 2, -8, -4, -2, -4, 0, 2, 0, 8, -8], [24, 9, -5, 9, 0, 9, 0, -14, 5, 12, 9, 0], [13, 0, 0, 0, 13, -5, 6, 6, 8, -2, 0, 2], [-13, -16, 4, -8, 13, 0, -5, 5, 0, -12, 0, -12], [5, 15, -4, 2, -5, 0, 4, -4, 0, 4, 0, 4], [-12, 0, 0, 0, -12, -16, 10, 10, -6, -8, -10, 8], [10, 10, -4, 3, 0, 10, 0, -10, 4, 10, 3, 0], [10, 3, 0, -2, 0, 3, 0, -2, 0, 12, -2, 0], [0, -14, -8, -12, 0, 0, -4, 4, 0, -2, 0, -2], [-6, 0, 0, 0, -6, 11, 1, 1, 0, 0, 8, 0], [0, -10, -8, 0, 8, 10, -8, 0, -8, 0, 0, -4], [-2, 5, -2, 4, 2, 0, -5, 5, 0, -6, 0, -6], [-2, 2, 7, 5, 0, 2, 0, -4, -7, 2, 5, 0], [5, 0, 0, 0, 5, 3, -9, -9, -8, 2, -10, -2], [11, 0, 0, 0, 11, -14, 6, 6, 6, -7, -12, 7], [-4, 7, 4, 9, 0, 7, 0, -8, -4, 2, 9, 0], [0, 6, -2, 2, -2, -6, 0, 0, -2, 0, -2, 14], [10, 0, 0, 0, 10, -5, -5, -5, 2, 0, 0, 0], [-7, 0, 0, 0, -7, 6, -8, -8, -10, -2, 6, 2], [-24, -1, -4, 3, 0, -1, 0, -2, 4, 10, 3, 0], [0, 3, 5, -1, -16, -3, 16, 0, 5, 0, 1, 2], [0, -6, -4, -4, 4, 6, -2, 0, -4, 0, 4, -4], [0, -3, -1, 1, 16, 3, -10, 0, -1, 0, -1, 8], [10, -7, -5, -6, 0, -7, 0, -2, 5, 0, -6, 0], [12, 3, -5, 5, 0, 3, 0, -12, 5, 2, 5, 0], [3, -3, 8, -6, -3, 0, -6, 6, 0, -10, 0, -10], [0, 0, -1, -5, 8, 0, 2, 0, -1, 0, 5, -2], [-27, -5, -4, 4, 27, 0, 0, 0, 0, 0, 0, 0], [-4, 11, -8, 8, 4, 0, -3, 3, 0, 4, 0, 4], [0, 1, 4, -2, -14, -1, -6, 0, 4, 0, 2, -10], [10, 0, 0, 0, 10, 8, -1, -1, 4, 7, 0, -7], [0, -12, -4, -8, -12, 12, -2, 0, -4, 0, 8, -4], [0, 0, 0, 0, 0, -7, 5, 5, 12, -4, 4, 4], [9, 0, 0, 0, 9, -13, -3, -3, -4, -6, 2, 6], [-8, -10, -2, 0, 0, -10, 0, 4, 2, 0, 0, 0], [0, -4, 12, -4, 12, 4, 16, 0, 12, 0, 4, -12]]
